@@ -4,9 +4,11 @@ use warnings;
 use strict;
 
 use File::Assets::Util;
+use File::Assets::Carp;
+use File::Assets::Asset::Content;
+
 use XML::Tiny;
 use IO::Scalar;
-use Carp::Clan qw/^File::Assets/;
 use Object::Tiny qw/type rank attributes hidden rsc/;
 
 =head1 SYNPOSIS 
@@ -59,23 +61,18 @@ sub new {
         $content = $tag->{content}->[0]->{content};
         $content = "" unless defined $content;
     }
-    $self->{content} = \$content;
-
-    my ($path, $rsc, $base) = delete @$asset{qw/path rsc base/};
-    my ($type) = delete @$asset{qw/type/};
 
+    my ($path, $rsc, $base, $type) = delete @$asset{qw/path rsc base type/};
     if (defined $type) {
         my $_type = $type;
         $type = File::Assets::Util->parse_type($_type) or croak "Don't understand type ($_type) for this asset";
     }
 
-    my ($key, $inline);
-    $inline = 0;
-
     if ($rsc) {
         croak "Don't have a type for this asset" unless $type;
         $self->{rsc} = $rsc;
         $self->{type} = $type;
+        croak "Can't also specify content and ", $self->rsc->file if defined $content;
     }
     elsif ($base && $path) {
         if ($path =~ m/^\//) {
@@ -84,26 +81,22 @@ sub new {
         else {
             $self->{rsc} = $base->child($path);
         }
+        croak "Can't also specify content and ", $self->rsc->file if defined $content;
         $self->{type} = $type || File::Assets::Util->parse_type($path) or croak "Don't know type for asset ($path)";
     }
     elsif (defined $content) {
-        $inline = 1;
         croak "Don't have a type for this asset" unless $type;
         $self->{type} = $type;
-    }
-    elsif (0 && $base && $content) { # Nonsense scenario?
-        croak "Don't have a type for this asset" unless $type;
-        my $path = File::Assets::Util->build_asset_path(undef, type => $type, digest => $self->digest);
-        $self->{rsc} = $base->child($path);
-        $self->{type} = $type;
+        $self->{digest} = File::Assets::Util->digest->add($content)->hexdigest;
+        $self->{content} = \$content;
     }
 
     my $rank = $self->{rank} = delete $asset->{rank} || 0;
     croak "Don't understand rank ($rank)" if $rank && $rank =~ m/[^\d\+\-\.]/;
-
-    $self->{mtime} = delete $asset->{mtime} || 0;
-    $self->{inline} = exists $asset->{inline} ? (delete $asset->{inline} ? 1 : 0) : $inline;
-
+    $self->{cache} = delete $asset->{cache};
+    $self->{inline} = exists $asset->{inline} ?
+        (delete $asset->{inline} ? 1 : 0) :
+        $self->{content} ? 1 : 0;
     $self->{attributes} = { %$asset }; # The rest goes here!
 
     return $self;
@@ -118,7 +111,7 @@ Returns a L<URI> object represting the uri for $asset
 sub uri {
     my $self = shift;
     return unless $self->{rsc};
-    return $self->rsc->uri;
+    return ($self->{uri} ||= $self->rsc->uri)->clone;
 }
 
 =head2 $asset->file 
@@ -142,32 +135,23 @@ Returns the path of $asset
 sub path {
     my $self = shift;
     return unless $self->{rsc};
-    return $self->rsc->path;
+    return $self->{path} ||= $self->rsc->path;
 }
 
 =head2 $asset->content 
 
-Returns a scalar reference to the content contained in $asset->file
+Returns a SCALAR reference to the content contained in $asset->file
 
 =cut
 
 sub content {
     my $self = shift;
-
-    if (my $file = $self->file) {
-        croak "Trying to get content from non-existent file ($file)" unless -e $file;
-        if (! $self->{content} || ($self->{mtime} != $file->stat->mtime)) {
-            local $/ = undef;
-            $self->{content} = \$self->file->slurp;
-            $self->{mtime} = $file->stat->mtime;
-        }
-    }
-    return $self->{content};
+    return $self->{content} || $self->_content->content;
 }
 
 =head2 $asset->write( <content> ) 
 
-Writes <content>, which should be a scalar reference, to the file located at $asset->file
+Writes <content>, which should be a SCALAR reference, to the file located at $asset->file
 
 If the parent directory for $asset->file does not exist yet, this method will create it first
 
@@ -194,9 +178,7 @@ Returns a hex digest for the content of $asset
 
 sub digest {
     my $self = shift;
-    return $self->{digest} ||= do {
-        File::Assets::Util->digest->add(${ $self->content })->hexdigest;
-    }
+    return $self->{digest} || $self->_content->digest;
 }
 
 sub content_digest {
@@ -205,33 +187,68 @@ sub content_digest {
     return $self->digest;
 }
 
-=head2 $asset->mtime
-
-Returns the (stat) mtime of $asset->file, or 0 if $asset->file does not exist
-
-=cut
-
 sub mtime {
+    return 0;
+    carp "File::Assets::Asset::mtime is DEPRECATED";
+}
+
+sub file_mtime {
     my $self = shift;
-    return $self->{mtime} unless $self->{rsc};
-    return 0 unless my $stat = $self->file->stat;
-    return $stat->mtime;
+    return 0 unless $self->file;
+    return $self->_content->file_mtime;
+}
+
+sub file_size {
+    my $self = shift;
+    return 0 unless $self->file;
+    return $self->_content->file_size;
+}
+
+sub content_mtime {
+    my $self = shift;
+    return 0 unless $self->file;
+    return $self->_content->content_mtime;
+}
+
+sub content_size {
+    my $self = shift;
+    return length ${ $self->{content} } unless $self->file;
+    return $self->_content->content_size;
+}
+
+sub refresh {
+    my $self = shift;
+    return 0 unless $self->file;
+    return $self->_content->refresh;
+}
+
+sub stale {
+    my $self = shift;
+    return 0 unless $self->file;
+    return $self->_content->stale;
+}
+
+sub _content {
+    my $self = shift;
+    return $self->{_content} ||= do {
+        if (my $cache = $self->{cache}) {
+            $cache->content($self->file);
+        }
+        else {
+            File::Assets::Asset::Content->new($self->file);
+        }
+    };
 }
 
 =head2 $asset->key
 
-Returns the unique key for the $asset. Usually the path of the $asset, but for content-based assets returns a value based off of $asset->digest
+Returns the unique key for the $asset. Usually the path/filename of the $asset, but for content-based assets returns a value based off of $asset->digest
 
 =cut
 
 sub key {
     my $self = shift;
-    if ($self->{rsc}) {
-        return $self->path;
-    }
-    else {
-        return $self->{key} ||= '%' . $self->digest;
-    }
+    return $self->path || ($self->{key} ||= '%' . $self->digest);
 }
 
 =head2 $asset->hide
@@ -259,3 +276,12 @@ sub inline {
 }
 
 1;
+
+__END__
+
+#    elsif (0 && $base && $content) { # Nonsense scenario?
+#        croak "Don't have a type for this asset" unless $type;
+#        my $path = File::Assets::Util->build_asset_path(undef, type => $type, digest => $self->digest);
+#        $self->{rsc} = $base->child($path);
+#        $self->{type} = $type;
+#    }
